/*-----------------------------------------------------------------------------
| Copyright (c) Jupyter Development Team, Spyder Project Contributors.
| Distributed under the terms of the Modified BSD License.
|----------------------------------------------------------------------------*/

body {
  background: var(--jp-layout-color1);
  margin: 0;
  padding: 0;
}

#main {
  position: absolute;
  top: 4px;
  left: 4px;
  right: 4px;
  bottom: 4px;
}

.jp-NotebookPanel {
  border-bottom: 1px solid var(--jp-border-color1);
}

.notebookMenuBar {
  border-bottom: 1px solid var(--jp-border-color0);
  min-height: 28px;
}
